@@ -8,4 +8,4 @@
 
 """
 
-from gitdiscribe import Gitdiscribe
+from .gitdiscribe import Gitdiscribe
