@@ -23,59 +23,67 @@ class PackFile(object):
     def __init__(self, git_dir_path, debug=False):
         self._debug = debug
         folder = os.path.join(git_dir_path, 'objects', 'pack')
+        self.idxs, self.packs = {}, {}
         if os.path.exists(folder):
-            pack_idx = glob.glob(os.path.join(folder, 'pack*.idx'))
-            if len(pack_idx) == 1:
-                self.idx = open(pack_idx[0], 'rb')
-            pack_pack = glob.glob(os.path.join(folder, 'pack*.pack'))
-            if len(pack_idx) == 1:
-                self.pack = open(pack_pack[0], 'rb')
+            for idx in glob.glob(os.path.join(folder, 'pack-*.idx')):
+                hsh = re.search('pack-([0-9a-f]{40})\.idx$', idx).group(1)
+                self.idxs[hsh] = open(idx, 'rb')
+                self.packs[hsh] = open(os.path.join(folder,
+                                      "pack-{}.pack".format(hsh)), 'rb')
+        if self.__bool__():
+            self.get_indizes()
         else:
             self.prtdb('shiet!')
             return None
-        if self.__bool__():
-            self.get_index()
 
-    def get_index(self):
-        import pprint
-        pp = pprint.PrettyPrinter(indent=4).pprint
-        magic, version = struct.unpack('>4sL', self.idx.read(8))
-        if not(magic == b'\xfftOc' and version == 2):
-            print('unknown file format')
-            self.prtdb(' {0}   {1}'.format(repr(magic), version))
-            self.idx, self.pack = None, None
-            return False
-        self.fanout = struct.unpack('>256L', self.idx.read(256 * 4))
-        self.noff = 258 * 4                               # where object names begin
-        self.crcoff = self.noff + self.fanout[255] * 20   # where crcs begin
-        self.offoff = self.crcoff + self.fanout[255] * 4  # where offsets begin
-        self.o64off = self.offoff + self.fanout[255] * 4  # where 8 byte offsets begin
+    def get_indizes(self):
+        self.poss = {}
+        for hsh,idx in self.idxs.items():
+            pos = self.poss[hsh] = {}
+            magic, version = struct.unpack('>4sL', idx.read(8))
+            if not(magic == b'\xfftOc' and version == 2):
+                print('unknown file format in {}'.format(hsh))
+                self.prtdb(' {0}   {1}'.format(repr(magic), version))
+                del(self.idx[hsh]); del(self.pack[hsh])
+                continue
+            pos['fanout'] = struct.unpack('>256L', idx.read(256 * 4))
+            pos['noff'] = 258 * 4                                   # where object names begin
+            pos['crcoff'] = pos['noff'] + pos['fanout'][255] * 20   # where crcs begin
+            pos['offoff'] = pos['crcoff'] + pos['fanout'][255] * 4  # where offsets begin
+            pos['o64off'] = pos['offoff'] + pos['fanout'][255] * 4  # where 8 byte offsets begin
 
     def find_offset(self, bid):
         self.prtdb(bid.sha_parts)
-        first = self.fanout[bid.d_int-1] if bid.d_int > 0 else 0
-        last = self.fanout[bid.d_int]
-        self.prtdb('first last: {0} - {1}'.format(first, last))
-        self.idx.seek(self.noff + first * 20)
-        for offset in range(first, last):
-            ntup = struct.unpack('>5L', self.idx.read(20))
-            self.prtdb('  {0}'.format(ntup))
-            if ntup == bid.sha_parts:
-                roffset = offset # the relative offset of the offset
+        for hsh,idx in self.idxs.items():
+            pos = self.poss[hsh]
+            first = pos['fanout'][bid.d_int-1] if bid.d_int > 0 else 0
+            last = pos['fanout'][bid.d_int]
+            self.prtdb('first last: {0} - {1}'.format(first, last))
+            idx.seek(pos['noff'] + first * 20)
+            for offset in range(first, last):
+                ntup = struct.unpack('>5L', idx.read(20))
+                self.prtdb('  {0}'.format(ntup))
+                if ntup == bid.sha_parts:
+                    roffset = offset # the relative offset of the offset
+                    fin_hsh = hsh
+                    fin_idx = idx
+                    break
+            if 'roffset' in locals():
                 break
         if not 'roffset' in locals():
             self.prtdb('FAILED to find offset for {0}'.format(bid))
-            return None
-        self.idx.seek(self.offoff + roffset * 4)
-        offset = struct.unpack('>L', self.idx.read(4))[0]
-        return offset
+            return None, None
+        fin_idx.seek(self.poss[fin_hsh]['offoff'] + roffset * 4)
+        offset = struct.unpack('>L', fin_idx.read(4))[0]
+        return fin_hsh, offset
 
     def read_object(self, bid):
-        ofs = self.find_offset(bid)
-        self.pack.seek(ofs)
+        hsh, ofs = self.find_offset(bid)
+        fh = self.packs[hsh]
+        fh.seek(ofs)
         bites = []
         while True:
-            bite = struct.unpack('>B', self.pack.read(1))[0]
+            bite = struct.unpack('>B', fh.read(1))[0]
             bites.append(bite)
             if 128 & bite != 128:
                 break
@@ -92,13 +100,13 @@ class PackFile(object):
         dcob = zlib.decompressobj()
         self.prtdb(repr(dcob.unused_data))
         while dcob.unused_data == b'':
-            out += dcob.decompress(self.pack.read(64))
+            out += dcob.decompress(fh.read(64))
         # prepending the object type and (inflated) length like in the object files
         return '{0} {1}\x00{2}'.format(typ, len(out), struni(out, 'utf-8'))
 
 
     def __bool__(self): # implicit call python 3 only
-        return hasattr(self, 'idx') and hasattr(self, 'pack')
+        return bool(len(self.idxs))
 
     def prtdb(self, matter):
         if self._debug:
